c***********************************************************************
      subroutine ehook(mw,iproc)
c***********************************************************************
c
c     This routine optionally accepts the mass-weighted cartesian 
c     coordinates and transforms them to cartesian coordinates (mw=1).
c     The energy is computed using the cartesian coordinates.  If the
c     mass-weighted option was chosen (mw=1), the coordinates are
c     transformed back to mass-weighted at the end.  
c
      implicit double precision (a-h,o-z)
c
      include 'param.inc'
      include 'percon.inc'
      include 'common.inc'
      include 'nwrate.inc'
      common /ncgcom/ iop
      dimension fmat(n3tm,n3tm)
      character*4 scftypej
c     
c     If it is a Grate restart, read the energy     
c     
      irout=1
      if (irest.eq.1) call grest(irout,dummy)
      if (irout.eq.0) return
c
c     Optionally convert the mass-weighted coordinates to cartesian
c
      if (mw.eq.1) call trans(2,n3,amass,x,dx)
c
c     Compute the potential using the appropriate routines
c
c     If the energy corresponds to a stationary point, check if it has
c     already been calculated and stored in redinf.
c
        if (iop.eq.6) then                                              1128JC97
            i=5
            nacat=natom
        else 
          if (iop.gt.0) then
            call redinf(iop,fmat,ierr)
            if (ierr.eq.0) goto 1
            i=iop
          else
            i=-iop
            iop=-iop
          endif
            if (iop.eq.5) then
                nacat=natom
            else
                nacat=nratom(iop)
            endif
        endif
c
c     If the point has not been stored, it calculates it
c
        ich=icharge(i)
        imul=imultipl(i)
        scftypej=scftype(i)
C Debug
        if(debug) then
          ntot=nratom(iop)
          do ii=1,ntot
           write(666,*)'IATSV: ',iatsv(ii,iop)
          enddo
        endif
C       call escener(natoms,ich,imul,natom,label,
C    *iatsv,v,x,iop,nacat,nratom)
        call escener(natoms,ich,imul,scftypej,natom,label,iatsv,        1004JZ07
     *v,x,iop,nacat,nratom,ivrc)                                        0921JZ09
c
c     Optionally transfer back to mass-weighted coordinates
1     if (mw.eq.1) call trans(1,n3,amass,x,dx)
c
c     Put the energy at an appropriate zero of energy
      if (lezer0) v = v-ezer0     
c
c     new format, the bath term is a function of X instead of S
c
      IF (LBATH) THEN
        CALL EFFBATH(0)
      ENDIF
c     
c     If it is a Grate restart, write the energy
c     
      if (irest.eq.1) then
          irout=5
          call grest(irout,dummy)
      endif
c
      return
      end                                           
c
c*****
c
c***********************************************************************
      subroutine ghook(mw,iproc)
c***********************************************************************
c
c     This routine optionally accepts the mass-weighted cartesian 
c     coordinates and transforms them to cartesian coordinates (mw=1).
c     The cartesian coordinates are used when computing the energy and
c     first derivatives of the energy with respect to each cartesian 
c     coordinate.  If the mass-weighted option was chosen (mw=1), the 
c     coordinates and first derivatives are transformed back to mass-
c     weighted at the end.  
c
c
      implicit double precision (a-h,o-z)
c
      include 'param.inc'
      include 'percon.inc'
      include 'common.inc'
      include 'nwrate.inc'
      common /ncgcom/ iop
      dimension fmat(n3tm,n3tm)
      character*4 scftypej
c     
c     If it is a Grate restart, read the energy and first der.     
c     
      irout=2
      if (irest.eq.1) call grest(irout,dummy)
      if (irout.eq.0) return
c
c     Optionally convert the mass-weighted coordinates to cartesian
c
      if (mw.eq.1) call trans(2,n3,amass,x,dx)
c
c     Compute the potential using the appropriate routines
c     If the calculation corresponds to a stationary point, check if it has
c     already been calculated and stored in redinf.
c
        if (iop.eq.6) then                                              1128JC97
            i=5
            nacat=natom
        else
            call redinf(iop,fmat,ierr)
            if (ierr.eq.0) goto 1
            i=iop
            if (iop.eq.5) then
                nacat=natom
            else
                nacat=nratom(iop)
            endif
        endif
c
c     If the point has not been stored, it calculates it
c
        ich=icharge(i)
        imul=imultipl(i)
        scftypej=scftype(i)
C       call escfirst(natoms,ich,imul,natom,label,
C    *iatsv,v,x,dx,iop,nacat,nratom)
        call escfirst(natoms,ich,imul,scftypej,natom,label,             1004JZ07
     *iatsv,v,x,dx,iop,nacat,nratom)
c
c     Put the energy at an appropriate zero of energy
c
1     if (lezer0) v = v-ezer0
c
c   calculate the derivative and energy, should be in Cartesian coord
c
      IF (LBATH) THEN
        CALL EFFBATH(1)
      ENDIF
c
c
c     Transfer back to mass-weighted coordinates
      if (mw.eq.1) call trans(1,n3,amass,x,dx)
c
c
c     If it is a Grate restart, write the energy and first der.
c
      if (irest.eq.1) then
          irout=6
          call grest(irout,dummy)
      endif
c
      return
      end                                           
c
c*****
c
c***********************************************************************
         subroutine ohook (iop,iproc)
c***********************************************************************
c
c     This routine optionally accepts a initial guess to the cartesian
c     coordinates of the full system and returns the optimized geometry
c     and energy
C
c     If NWChem is optimizing, all the optimization has been carried out 
c     in prep. Therefore, this subroutine only reads the and prints out
c     the results of the calculations in prep
c
c     The geometry is optimized for a given chemical species:
c         For IOP = 1, or 2; the chemical species is reactant 1 or 2.
c         For IOP = 3, or 4; the chemical species is product 1 or 2.
c         For IOP = 5, the chemical species is the saddle point.
c         For IOP = 6, or 7; the chemical species is reactants or 
c                            products wells.
c
      implicit double precision (a-h,o-z)
c
      include 'param.inc'
      include 'percon.inc'
      include 'common.inc'
      include 'nwrate.inc'
c     dimension fmat(n3tm,n3tm)                                         0701TA03
c     Variable fmat was not used so it was taken out in Gaussrate 9.1.  0701TA03
c     
c     if Polyrate is going to do the optimization, call newt.
c
      if (igpot.eq.0) then
                 call gsetp(iop)
                 if (nratom(iop).gt.1) then
                    call newt(iop)
                 else 
                    call ehook(0,iproc)
                 endif
                 if (lezer0) v = v-ezer0
                 return
      endif
      if (isup.eq.1) call gsetp(iop)
c
c     else:
c     If it is a NWRate restart, read the geom, energ and 1st der.
c     
      irout=4
      if (irest.eq.1) call grest(irout,dummy)
      if (irout.eq.0) goto 3
c
      if (iop.le.2) then
           v=enresv
            if (isup.eq.1) then
                if (iop.eq.1) v=ereac1
                if (iop.eq.2) v=ereac2
            endif
           do m=1,n3tm
              x(m)=geresv(m)
             dx(m)=fsresv(m)
           enddo
      else if (iop.le.4) then
           v=enprsv
            if (isup.eq.1) then
                if (iop.eq.3) v=eprod1
                if (iop.eq.4) v=eprod2
            endif
           do m=1,n3tm
              x(m)=geprsv(m)
             dx(m)=fsprsv(m)
           enddo
      else if (iop.eq.7) then                                           1128JC97
           v=enwrsv                                                     1128JC97
           do m=1,n3tm                                                  1128JC97
              x(m)=gewrsv(m)                                            1128JC97
             dx(m)=fswrsv(m)                                            1128JC97
           enddo                                                        1128JC97
      else if (iop.eq.8) then                                           1128JC97
           v=enwpsv                                                     1128JC97
           do m=1,n3tm                                                  1128JC97
              x(m)=gewpsv(m)                                            1128JC97
             dx(m)=fswpsv(m)                                            1128JC97
           enddo                                                        1128JC97
      else
           v=enstsv
           do m=1,n3tm
              x(m)=gestsv(m)
             dx(m)=fsstsv(m)
           enddo
      endif
c    
c     Print out the results of the NWChem optimization
c
c     Geometry
c
3     if (iop.eq.5) write(fu6,1006)
c     write (fu6,1001)
      if (iunit6.eq.1) write(fu6,1001)                                  0703JZ07
      if (iunit6.eq.0) write(fu6,1002)                                  0703JZ07
      write (fu6,1011)
      inat=1
1     if (iop.ne.5) then
         inum = iatsv(inat,iop)
      else 
         inum = inat
      endif
c        write(fu6,1021)inum,(x(3*(inum-1)+j), j=1,3)
         write(fu6,1021)inum,(x(3*(inum-1)+j)/gufac6, j=1,3)            0703JZ07
      inat=inat+1
      if (iop.eq.5) then                                                1128JC97
           if (inat.le.natom) goto 1
      else
           if (iatsv(inat,iop).ne.0) goto 1
      endif
      write (fu6,1011)
c
c     Gradients
c
      write (fu6,1031)
      write (fu6,1011)
      inat=1
2     if (iop.ne.5) then                                                0615JC98
         inum = iatsv(inat,iop)
      else
         inum = inat
      endif
         write(fu6,1021)inum,(dx(3*(inum-1)+j), j=1,3)
      inat=inat+1
      if (iop.eq.5) then                                                1128JC97
           if (inat.le.natom) goto 2
      else
           if (iatsv(inat,iop).ne.0) goto 2
      endif
      write (fu6,1011)
c
c   calculate the derivative and energy, should be in Cartesian coord
c
      IF (LBATH) THEN
        CALL EFFBATH(1)
      ENDIF
c
c
c
c     If it is a NWRate restart, write the geom., energy and 1st der.
c     
      if (irout.eq.0) return
      if (irest.eq.1) then
          irout=8
          call grest(irout,dummy)
      endif
      return
1001     format(/4x,'Geometry optimized using NWChem (a.u.)')
1002     format(/4x,'Geometry optimized using NWChem (angstrom)')
1006     format(//1X,32(1H*),' Saddle point ',32(1H*)//)
1011     format(/4x,55('-'),/)
1021     format(4x,i2,8x,3(f11.6,4x))
1031     format(/4x,'Gradients of the optimized structure (a.u.)')
      end                                           
c
c*****
c
c***********************************************************************
        SUBROUTINE nwinpfl(ntype,ICHARGEJ,IMULTIPLJ,SCFTYPEJ,NATOM,LABEL
     *  ,iatsv,COORD,jtype,nacat,NATESC,NRATOM)
c***********************************************************************
c
c     This subroutine writes the input file for NWChem
c       ntype determines type of calculation
c         =1: energy
c         =2: energy+gradient
c         =3: energy+gradient+hessian
c         =4: geometry optimisation+energy+gradient+hessian
c         =5: transition state optimisation+energy+gradient+hessian
c     This subroutine is a modified version of Gaussrate
c     ginpene, modified by MAI Nov. 10, 2006
c
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
      include 'param.inc'
      include 'percon.inc'
      include 'nwrate.inc'
      DIMENSION COORD(N3TM)
      DIMENSION LABEL(NATESC),IATSV(NATESC,8),NRATOM(8)
      character*80 nwtask
      character*2 clab
      character*4 scftypej
C
C
      hatom=.false.
      OPEN (UNIT=FU81,STATUS='unknown',FILE='esp.fu81')
C
c   Write the 'top' part of the file
C
       if (nlntop.gt.0)then
         do ii=1,nlntop
           WRITE(FU81,'(a80)')nwtop(ii)
         enddo
       endif
C
C   Write the charge of the system
C
      write(fu81,1001)ICHARGEJ
C
C   Write the methods options section
C
c      if((nwmeth(1:3).eq.'dft').or.
c     *     (nwmeth(1:5).eq.'sodft'))then
      if(index(nwmeth,'dft').ne.0)then
        write(fu81,*)nwmeth
        if(nlnmethop.gt.0)then
           do ii=1,nlnmethop
              write(fu81,'(a80)')nwmethop(ii)
           enddo
        endif
        if(IMULTIPLJ.ne.1)write(fu81,1002)IMULTIPLJ
        write(fu81,*)'end'
        write(fu81,*)
      elseif(index(nwmeth,'scf').ne.0)then
         write(fu81,*)'scf'
            if(nlnmethop.gt.0)then
               do ii=1,nlnmethop
                  write(fu81,'(a80)')nwmethop(ii)
               enddo
            endif
            if(IMULTIPLJ.eq.1.)write(fu81,*)'singlet'
            if(IMULTIPLJ.eq.2.)write(fu81,*)'doublet'
            if(IMULTIPLJ.eq.3.)write(fu81,*)'triplet'
            if(IMULTIPLJ.eq.4.)write(fu81,*)'quartet'
            if(IMULTIPLJ.eq.5.)write(fu81,*)'quintet'
            if(IMULTIPLJ.eq.6.)write(fu81,*)'sextet'
            if(IMULTIPLJ.eq.7.)write(fu81,*)'septet'
            if(IMULTIPLJ.eq.8.)write(fu81,*)'octet'
            if(IMULTIPLJ.ge.9.)write(fu81,*)'nopen ',IMULTIPLJ
            write(fu81,*)'end'
            write(fu81,*)
      else
         write(fu81,'(a3)')'scf'
         if(nlnscf.gt.0)then
            do ii=1,nlnscf
              write(fu81,'(a80)') nwscfop(ii)
            enddo
         endif
         write(fu81,*) scftypej                                         1004JZ07
         if(IMULTIPLJ.eq.1.) then
            write(fu81,*)'singlet'
         elseif(IMULTIPLJ.eq.2.) then
            write(fu81,*)'doublet'
         elseif(IMULTIPLJ.eq.3.) then
            write(fu81,*)'triplet'
         elseif(IMULTIPLJ.eq.4.) then
            write(fu81,*)'quartet'
         elseif(IMULTIPLJ.eq.5.) then
            write(fu81,*)'quintet'
         elseif(IMULTIPLJ.eq.6.) then
            write(fu81,*)'sextet'
         elseif(IMULTIPLJ.eq.7.) then
            write(fu81,*)'septet'
         elseif(IMULTIPLJ.eq.8.) then
            write(fu81,*)'octet'
         elseif(IMULTIPLJ.ge.9.) then 
            write(fu81,*)'nopen ',IMULTIPLJ
         else
            write(fu6,*)'Multiplicity is wrong'
            stop 'writing input for SCF'
         endif                                                          1004JZ07
C           if(mod(IMULTIPJ,2).eq.0)then
C              write(fu81,*)'RHF'
C           else
C              write(fu81,*)'ROHF'
C           endif
         write(fu81,'(a3)') 'end'
         write(fu81,*)
C If the species is Hydrogen atom, the post-HF methods will not         1004JZ07
C be used
        If(jtype.le.4) then                                             1004JZ07
            ntot= nratom(jtype)
            if(debug) write(666,*)'jtype: ',jtype,' ntot: ',ntot
            if(ntot.eq.1) then
              inum = iatsv(1,jtype)
            if(debug) write(666,*)'INUM= ',inum
              ilab = label(inum)
              if(ilab.eq.1) then
                hatom=.true.
                goto 100
              endif
            endif
        endif       
C
        write(fu81,'(a10)')nwmeth
        if(nlnmethop.gt.0)then
           do ii=1,nlnmethop
              write(fu81,'(a80)')nwmethop(ii)
           enddo
        endif
        write(fu81,'(a3)') 'end'
        write(fu81,*)
100     continue
      endif
C
C   Write atomic symbols and coordinates
C
C   Write the needed options for the geometry block
C      units au  => use a.u. instead of angstroms as per POLYRATE standard
C      noautosym & nocenter => prevent reorientation of the molecule and
C                              maintains user rather than standard orientation
      write(fu81,*)'geometry units au noautosym nocenter'
      i=1
1        if (jtype.ne.5.and.jtype.ne.6) then                            1128JC97
            inum = iatsv(i,jtype)
         else
            inum = i
         end if
         clab = elem(label(inum))
         write(FU81,1021)clab,(coord(3*(inum-1)+j), j=1,3)
         if(debug)write(666,1021)clab,(coord(3*(inum-1)+j), j=1,3)
         if(debug)write(666,*)'inum= ',inum
      i=i+1
      if (jtype.eq.5.or.jtype.eq.6) then                                1128JC97
               if (i.le.natom) then
                     goto 1
               endif
      else
               if (iatsv(i,jtype).ne.0) then
                     goto 1
               endif
      endif
      WRITE(FU81,*)'end'
      write(fu81,*)
C
c   Write the 'middle' part of the NWChem file
C
      if(nlnmid.gt.0)then
         do ii=1,nlnmid
	    write(fu81,'(a80)')nwmid(ii)
	 enddo
      WRITE(FU81,*)'end'
      write(fu81,*)
      endif
C
C    Write the required task command
C
c      icend=len_trim(nwmeth)
c      call rword(string,icstrt,ierr)
c      nwtask='task '//nwmeth(icstart+1:icend)
      nwtask='task '//nwmeth
      if(ntype.eq.1)then
         nwtask='task '//nwmeth//' energy'
         if(hatom)nwtask='task scf energy'
	 write(fu81,*)nwtask
      elseif(ntype.eq.2)then
         nwtask='task '//nwmeth//' gradient'
	 write(fu81,*)nwtask
      elseif(ntype.eq.3)then
         nwtask='task '//nwmeth//' hessian'
	 write(fu81,*)nwtask
         nwtask='task '//nwmeth//' gradient'
	 write(fu81,*)nwtask
      elseif(ntype.eq.4)then
         nwtask='task '//nwmeth//' optimize'
	 write(fu81,*)nwtask
      elseif(ntype.eq.5)then
         nwtask='task '//nwmeth//' saddle'
	 write(fu81,*)nwtask
      else
         write(fu6,1100)ntype
	 stop
      endif
      write(fu81,*)
      write(fu81,*)

      CLOSE (FU81)
C
      RETURN
C
 1001  FORMAT (/,'charge ',I2,/)
 1002  FORMAT ('mult ',I2)
 1011  FORMAT (1x,I3,1x,I3)
 1021  FORMAT (1x,A2,5x,3(f14.8,3x))
 1031  FORMAT (/,$)
 1100  format('Error: unusual ntype ,',I4,' passed to nwinpfl.')
C
         END
c
c*****
c
c***********************************************************************
      SUBROUTINE rdnwout (IOP,V,COORD,TMPDX,FMAT,JTYPE,IATSV,
     *NATOM,NACAT,NATESC,NRATOM,LABEL,IVRC)
c***********************************************************************
C
C  Subroutine that extracts the geometry, energy, first and second 
C  derivatives information from the NWChem output file.
C  Inspired by Gaussrate goutsec routine, but writen from scratch
C  by MAI Nov. 10, 2006
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      include 'mpif.h'
      include 'param.inc'
      include 'percon.inc'
      include 'nwrate.inc'
C
      DIMENSION COORD(N3TM),FMAT(N3TM,N3TM),LABEL(NATESC),NRATOM(8)
      DIMENSION TMPFMAT(N3TM,N3TM), IATSV(NATESC,8)
      DIMENSION TMPFM(N3TM*N3TM), TMPDX(N3TM)
      CHARACTER*80 line
      character*17 srche
      character*19 srchg
      character*24 srcho
      character*16 srchf
      character*2 atsym
      character*10 ctemp
      character*80 case,ucase
      external case,ucase
      logical done,enfnd,optfnd,grdfnd
      integer lsrche,lsrchg
c
c     IOP = 0 obtain energy
c     IOP = 1 obtain energy and geometry
c     IOP = 2 obtain energy, geometry, and gradient
c     IOP = 3 obtain energy, geometry, gradient, and hessian
c
      do i=1,n3tm
        tmpdx(i)=0
      enddo
      do i=1,n3tm*n3tm
        tmpfm(i)=0
      enddo

      open(unit=fu85,status='unknown',file='esp.fu85')
C Check if the species is Hydrogen atom
      hatom=.false.
        If(jtype.le.4.and.index(nwmeth,'dft').eq.0) then                1004JZ07
            ntot= nratom(jtype)
            if(debug) write(666,*)'jtype: ',jtype,' ntot: ',ntot
            if(ntot.eq.1) then
              inum = iatsv(1,jtype)
              ilab = label(inum)
              if(ilab.eq.1) then
                hatom=.true.
              endif
            endif
        endif       
C
c
c Find the energy
c
      icend=len_trim(nwmeth)
      call rword(nwmeth,icstrt,ierr)
      srche=nwmeth(icstart+1:icend)//':energy'
      if(hatom)srche='scf:energy'
      lsrche=len_trim(srche)
      srchg=nwmeth(icstart+1:icend)//':gradient'
      lsrchg=len_trim(srchg)
      srcho='geometry:geometry:coords'
      srchf='Status    = fail'

      if(debug)write(fu6,*)'**',srche(1:lsrche),'**'
      if(debug)write(fu6,*)'**',srchg(1:lsrchg),'**'
      if(debug)write(fu6,*)'**',srcho,'**'
      
      done=.false.
      enfnd=.true.
      optfnd=.true.
      grdfnd=.true.
      if(iop.lt.2)grdfnd=.false.
      if(iop.lt.1)optfnd=.false.
c      if(NATOM.eq.1)grdfnd=.false.
      
      do while(.not.done)
         read(fu85,'(a)',end=9999)line
         if(index(line,srche(1:lsrche)).ne.0)then
            if(debug)write(666,*)line
            read(fu85,*)V
            enfnd=.false.
         elseif((index(line,srchg(1:lsrchg)).ne.0).and.(iop.ge.2))then
            if(debug)write(666,*)line
            read(fu85,*)(tmpdx(i),i=1,3*NACAT)
            grdfnd=.false.
         elseif((index(line,srcho).ne.0).and.(iop.ge.1))then
            if(debug)write(666,*)line
            read(fu85,*)(coord(i),i=1,3*NACAT)
            optfnd=.false.
         elseif(index(line,srchf).ne.0)then
            if(ivr.eq.1) then
               V=1.0d05
               enfnd=.false.
               goto 9999
            else
              write(fu6,6666)
              stop 'rdnwout 1' 
            endif
         endif
      enddo
 9999 continue
 
      if(debug)call flush_(666)

      if(enfnd.or.grdfnd.or.optfnd)then
         if(debug)write(666,*)enfnd,grdfnd,optfnd
         write(fu6,6666)
         stop 'rdnwout 2'
      endif

      close(fu85)

      if(iop.eq.3)then
         open(unit=fu86,status='unknown',file='esp.fu86')
         do i=1,((NACAT*3)*(NACAT*3-1))/2+NACAT*3
           read(fu86,*)TMPFM(i)
         enddo
         close(fu86)
c Convert vector of triangular force constants TMPFM to Hessian maxtrix TMPFMAT
         k=1
         do 1 INX=1,3*NACAT
         do 1 INY=1,INX
            TMPFMAT(INX,INY)=TMPFM(k)
            IF(INX.ne.INY)TMPFMAT(INY,INX)=TMPFMAT(INX,INY)
            k=k+1
 1       continue
c Reorder Hessian according to the order of the atoms at the saddle point
         if((jtype.ne.5).and.(jtype.ne.6))then
            do 2 l=1,nacat
            do 2 m=1,nacat
               do 3 i=1,3
               do 3 j=1,3
                  innx=3*(iatsv(l,jtype)-1)+i
                  inny=3*(iatsv(m,jtype)-1)+j
                  inox=3*(l-1)+i
                  inoy=3*(m-1)+j
                  FMAT(innx,inny)=TMPFMAT(inox,inoy)
 3             continue
 2          continue
          else
            do 4 innx=1,n3tm
            do 4 inny=1,n3tm
               FMAT(innx,inny)=TMPFMAT(innx,inny)
 4          continue
         endif
      endif


 6666 format('NWChem job apparently failed - please check.')
      end
c
c*****
c
c***********************************************************************
       subroutine grest(irout,fmat)
c***********************************************************************
C
C    This subroutine compares the geometry with the one in the file and
C    if they are the same, it reads the properties of the point, so that
C    it will not be calculated again by means of Gaussian.
C    If the properties were stored, it returns irout=0
C
      implicit double precision (a-h,o-z)
      include 'param.inc'
      include 'percon.inc'
      include 'common.inc'
      include 'nwrate.inc'
      common /ncgcom/ iop
      common /nccmcm/ iopc
      dimension fmat(n3tm,n3tm)
C     fallout=1.0D-8                                                    0825JC00
      icontrw=0
C
C     The firt thing we have to do is tp check if we only want to write
C     (then, irout>4) or we want reading and writing. In the last case:
C
1     if (irout.le.4) then
C
C     First, if the previous comparison was ok, we read everything and put 
C     it on an array for the next comparisons
C
        read (fu83,*,END=999) irread
        if (irread.eq.4) read(fu83,*,END=999)ip,icharge(ip),imultipl(ip)
        read (fu83,*,END=999) vread
        do icount=1,3*natom
        read (fu83,*,END=999) georead(icount)
        enddo
        do icount=1,3*natom
        read (fu83,*,END=999) dxread(icount)
        enddo
        if (irread.eq.3) then
        do icount=1,3*natom
        do jcount=1,3*natom
          read (fu83,*,END=999) fmread(icount,jcount)
        enddo
        enddo
      endif
C
C    If the stored point does the same calculation that we want to do,
C    continue. Otherwise, the program looks for a place where it is done
C
        if (irout.ne.irread) then
               if (icontrw.eq.0) then
                  icontrw=1
                  rewind fu83
                  goto 1
               else
                  goto 1
               endif
        endif
C
C    If it is a geometry optimization, it reads geometry, energy and 1st der.
C
        if (irout.eq.4) then
          v=vread
          do icount=1,natom*3
             x(icount)=georead(icount)
          enddo
          do icount=1,natom*3
             dx(icount)=dxread(icount)
          enddo
C
c   and now writes is to gwrite
c
          write (fu84,*) irread
          if (irread.eq.4) write(fu84,*)ip,icharge(ip),imultipl(ip)
          write (fu84,*) v
          do icount=1,3*natom
          write (fu84,*) x(icount)
          enddo
          do icount=1,3*natom
          write (fu84,*) dx(icount)
          enddo
c   
c   and sets irout=0 to indicate that everything was ok.
c       
          irout=0
          return
        else
c
C    Now, if it wasn't a geometry optimization, compare the geometres. 
C    If the one to be used in the calculations is different from the one in 
C    gwrite, it returns (it doesn't allow rotations or translations in order
C    to avoid problems along the reaction path). If IOP<5 (i.e., before
C    enetring the reaction path calculations) the comparison between
C    geometries is not performed. It is assumed that for reactants,
C    products and saddle point everything is exactly the same than
C    in the original calculation
C    The same for wells (IOP>6)
C
          if (iop.eq.6) then                                            1128JC97
          do icount=1,natom*3
            diff=x(icount)-georead(icount)
            if (abs(diff).gt.fallout) then  
               if (icontrw.eq.0) then
                  icontrw=1
                  rewind fu83
                  goto 1
               else
                  goto 1
               endif
            endif
          enddo
          endif
C          
C    If both geometries are the same, it reads the stored information
C    and writes to gwrite
C
          v=vread
          if (irout.gt.1) then
            do icount=1,natom*3
               dx(icount)=dxread(icount)
            enddo
          endif
          if (irout.eq.3) then
            do icount=1,natom*3
             do jcount=1,natom*3
               fmat(icount,jcount)=fmread(icount,jcount)
             enddo
            enddo
          endif
        write (fu84,*) irread
        write (fu84,*) vread
        do icount=1,3*natom
        write (fu84,*) georead(icount)
        enddo
        do icount=1,3*natom
        write (fu84,*) dxread(icount)
        enddo
        if (irout.eq.3) then
        do icount=1,3*natom
        do jcount=1,3*natom
         write (fu84,*) fmread(icount,jcount)
        enddo
        enddo
        endif
c   
c   and sets irout=0 to indicate that everything was ok.
c       
          irout=0
      endif
      return
c
c     if irout>4 (i.e., we only want writing):
c
      else
          irout=irout-4
          write (fu84,*) irout
          if (irout.eq.4) write(fu84,*)iop,icharge(iop),imultipl(iop)
          write (fu84,*) v
          do icount=1,3*natom
               write (fu84,*) x(icount)
          enddo
          do icount=1,3*natom
               write (fu84,*) dx(icount)
          enddo
          if (irout.eq.3) then
             do icount=1,3*natom
                do jcount=1,3*natom
                   write (fu84,*) fmat(icount,jcount)
                enddo
             enddo
           endif
      endif
999   if (icontrw.eq.0) then
         icontrw=1
         rewind fu83
         goto 1
      else
         RETURN
      endif
      RETURN
      END
c
c*****
c
c***********************************************************************
      subroutine prep
c***********************************************************************
c
c     This routine is the driver for all preparation routines needed 
c     to set up a gaussrate calculation.
c
c     This routine is called after all POLYRATE input has been read in
c     and before the table of input is written to fu6.  It is also
c     before any calculations are done.  This routine is used
c     to call all electronic structure preparation routines. 
c
      implicit double precision (a-h,o-z)
c
      include 'param.inc'
      include 'percon.inc'
      include 'common.inc'
      include 'nwrate.inc'
      common /ncgcom/ iop
      common /nccmcm/ iopc
      common /eshscm/ fmat(n3tm,n3tm)
      dimension xcm(3)
      dimension nrs(8)
c
c     Check if it is the first call to this subroutine
c
      if (numci.eq.0) then
c
c     if it is, set up some variables and start the NWChem
c     optimization procedure (if selected) and the hessian 
c     calculations for reactant and products.
c
      if(debug.or.debug2)open(666,status='unknown',file='esp.fu666')
      if(debug.or.debug2)write(666,*)'Debug file'

      ereac=0.D0
      ereac1=0.D0
      ereac2=0.D0
      eprod=0.D0
      eprod1=0.D0
      eprod2=0.D0
      do i=1,8
        nrs(i)=nratom(i)
      enddo
      nrs(5)=natom
      do i=1,8
        vjdep(i)=0.D0
        icharge(i)=0
        imultipl(i)=1
        ioptgr(i)=0
      enddo
      call setup
c
c     Set isavect. If it is set to zero, the computer time is reduced as
c     much as possible, but it can be dangerous for optimizing geometries,
c     calculating anharmonicities, and second (or higher) derivatives by means 
c     of Polyrate methods. If it is set to one, some calculations may be 
c     repeated, but it will be much safer for the above mentioned
c     calculations. If isavect=2, the calculation is completely safe,
c     no check for saving time is done and derivatives will be calculated
c     correctly.
c     isavect=1 has been removed, since it saves little computer time
c     and there is a certain risk of failure
c
      isavect=0
      do 3 inesp=1,8                                                    1128JC97
       if (inesp.eq.6) goto 3                                           1128JC97
       if (ispot.eq.0) isavect=3
       if (igpot.eq.0) then
        if (istatu(inesp).eq.0) then                                    0729YC97
               if (nrs(inesp).gt.1) then
                     isavect=2
                     goto 1
               endif
        endif
       endif
        if (iharm(inesp).ne.1) then
            do inm=1,nxmod(inesp)
             if (vharmr(inm,inesp).ne.'harmonic '.and.
     *           vharmr(inm,inesp).ne.'hinrot   ') then
                       isavect=3
             endif
            enddo
        endif
3     continue
c
c
1     numci=1 
      if (isavect.eq.2) then
             if (isup.eq.1) return
             if (isup.eq.0) then
                    write (FU6,1001)
                    stop 'prep 1'
             endif
      endif
      if (isavect.eq.3) then
             if (isup.eq.1) isavect=2
             if (isup.eq.0) then
                    write (FU6,1001)
                    stop 'prep 2'
             endif
      endif
C
C     Calculations for reactants
C
      if (istatu(1).ne.6.or.istatu(2).ne.6) then                        0725YC97
      do iopc = 1,2                                          
        if (irepr(iopc).ne.0) then
            if (istatu(iopc).ne.6) then                                 0729YC97
            if (nratom(iopc).gt.1) then
              if (ioptgr(iopc).eq.1.and.istatu(iopc).lt.2) then         0725YC97
                 call geoin(iopc,ircm)
                 if (ircm.eq.1) call rchmul(iopc)
              else
                 imultipl(iopc)=nedeg(3*iopc-2)                         0725YC97
                 do i=1,nratom(iopc)                                    0729YC97
                     inum = iatsv(i,iopc)                               0729YC97
                     do j = 2,0,-1                                      0729YC97
                        x(inum*3-j) = xr(inum*3-j,1)                    0729YC97
                     enddo                                              0729YC97
                 enddo                                                  0729YC97
              endif
                 if (isavect.ne.2) then
                   call secin(iopc)
                 else
                   itmp=iop
                   iop=iopc
                   call ehook(0,iproc)
                   iop=itmp
                 endif
            else
              if (ioptgr(iopc).eq.1) then
                 call enatin(iopc,ircm)
                 if (ircm.eq.1) call rchmul(iopc)
              else
                 itmp=iop
                 iop=-iopc
                 call ehook(0,iproc)
                 iop=itmp
              endif
            endif
            else                                                        0729YC97
              V = steng(iopc)                                           0729YC97
              do i=1,nratom(iopc)                                       0729YC97
              inum = iatsv(i,iopc)                                      0729YC97
                 do j = 2,0,-1                                          0729YC97
                    x(inum*3-j) = xr(inum*3-j,1)                        0729YC97
                 enddo                                                  0729YC97
              enddo                                                     0729YC97
            endif                                                       0729YC97
c         vjdep(iopc)=v
        ereac=ereac+V
        if (isup.eq.1) then
            if (iopc.eq.1) then                                         1201JC97
               ereac1=V
            else
               ereac2=V
            endif
        endif
        endif
      enddo
C
C     saves the result for the following calculations on these points
C
        if (isup.eq.0) then
            itmp=iop
            iop=5
            call ehook(0,iproc)
            iop=itmp
            ereac=v
        endif
       enresv=ereac
       if (isup.eq.0) ezer0=ereac
       do i=1,n3tm
       geresv(i)=x(i)
       xr(i,1)=x(i)
       x(i)=0.D0
       fsresv(i)=DX(i)
       DX(i)=0.D0
        do j=1,n3tm
        hsresv(i,j)=fmat(i,j)
        fmat(i,j)=0.d0
        enddo
       enddo
      endif                                                             0725YC97
C
C     Calculations for products
C
      if (istatu(3).ne.6.or.istatu(4).ne.6) then                        0725YC97
      do iopc = 3,4
        if (irepr(iopc).ne.0) then
            if (istatu(iopc).ne.6) then                                 0729YC97
            if (nratom(iopc).gt.1) then
              if (ioptgr(iopc).eq.1.and.istatu(iopc).lt.2) then         0725YC97
                 call geoin(iopc,ircm)
                 if (ircm.eq.1) call rchmul(iopc)
              else
                 imultipl(iopc)=nedeg(3*iopc-2)                         0725YC97
                 do i=1,nratom(iopc)                                    0729YC97
                     inum = iatsv(i,iopc)                               0729YC97
                     do j = 2,0,-1                                      0729YC97
                        x(inum*3-j) = xr(inum*3-j,3)                    0729YC97
                     enddo                                              0729YC97
                 enddo                                                  0729YC97
              endif
                 if (isavect.ne.2) then
                 call secin(iopc)
                 endif
            else
              if (ioptgr(iopc).eq.1) then
                 call enatin(iopc,ircm)
                 if (ircm.eq.1) call rchmul(iopc)
              else
                 itmp=iop
                 iop=-iopc
                 call ehook(0,iproc)    
                 iop=itmp
              endif
            endif
            else                                                        0729YC97
              V = steng(iopc)                                           0729YC97
              do i=1,nratom(iopc)                                       0729YC97
              inum = iatsv(i,iopc)                                      0729YC97
                 do j = 2,0,-1                                          0729YC97
                    x(inum*3-j) = xr(inum*3-j,3)                        0729YC97
                 enddo                                                  0729YC97
              enddo                                                     0729YC97
            endif                                                       0729YC97
c         vjdep(iopc)=v
      eprod=eprod+v
        if (isup.eq.1) then
            if (iopc.eq.3) then                                         1201JC97
               eprod1=V
            else
               eprod2=V
            endif
        endif
        endif
      enddo
        if (isup.eq.0) then
            itmp=iop
            iop=5
            call ehook(0,iproc)
            iop=itmp
            eprod=v
        endif
C
C     saves the result for the following calculations on these points
C
       enprsv=eprod
       do i=1,n3tm
       geprsv(i)=x(i)
       xr(i,3)=x(i)
       x(i)=0.D0
       fsprsv(i)=DX(i)
       dx(i)=0.D0
        do j=1,n3tm
        hsprsv(i,j)=fmat(i,j)
        fmat(i,j)=0.D0
        enddo
       enddo
      endif                                                             0725YC97
C
C     Calculations for wells                                            1128JC97
C
      if (istatu(7).ne.6.or.istatu(8).ne.6) then                        1128JC97
       do iopc = 7,8                                                    1128JC97
        if (irepr(iopc).ne.0) then                                      1128JC97
            if (istatu(iopc).ne.6) then                                 1128JC97
              if (ioptgr(iopc).eq.1.and.istatu(iopc).lt.2) then         1128JC97
                 call geoin(iopc,ircm)                                  1128JC97
                 if (ircm.eq.1) call rchmul(iopc)                       1128JC97
              else                                                      1128JC97
                 imultipl(iopc)=nedeg(3*iopc-2)                         1128JC97
                 do i=1,nratom(iopc)                                    1128JC97
                     inum = iatsv(i,iopc)                               1128JC97
                     do j = 2,0,-1                                      1128JC97
                        x(inum*3-j) = xr(inum*3-j,iopc)                 1128JC97
                     enddo                                              1128JC97
                 enddo                                                  1128JC97
              endif                                                     1128JC97
                 if (isavect.ne.2) then                                 1128JC97
                    call secin(iopc)                                    1128JC97
                 endif                                                  1128JC97
            else                                                        1128JC97
              V = steng(iopc)                                           1128JC97
              do i=1,nratom(iopc)                                       1128JC97
              inum = iatsv(i,iopc)                                      1128JC97
                 do j = 2,0,-1                                          1128JC97
                    x(inum*3-j) = xr(inum*3-j,iopc)                     1128JC97
                 enddo                                                  1128JC97
              enddo                                                     1128JC97
            endif                                                       1128JC97
        endif                                                           1128JC97
C
C     saves the result for the following calculations on these points
C
        if (iopc.eq.7) then                                             1128JC97
           enwrsv=V                                                     1128JC97
           do i=1,n3tm                                                  1128JC97
              gewrsv(i)=x(i)                                            1128JC97
              xr(i,7)=x(i)                                              1128JC97
              x(i)=0.D0                                                 1128JC97
              fswrsv(i)=DX(i)                                           1128JC97
              dx(i)=0.D0                                                1128JC97
              do j=1,n3tm                                               1128JC97
                hswrsv(i,j)=fmat(i,j)                                   1128JC97
                fmat(i,j)=0.D0                                          1128JC97
              enddo                                                     1128JC97
           enddo                                                        1128JC97
        else                                                            1128JC97
           enwpsv=V                                                     1128JC97
           do i=1,n3tm                                                  1128JC97
              gewpsv(i)=x(i)                                            1128JC97
              xr(i,7)=x(i)                                              1128JC97
              x(i)=0.D0                                                 1128JC97
              fswpsv(i)=DX(i)                                           1128JC97
              dx(i)=0.D0                                                1128JC97
              do j=1,n3tm                                               1128JC97
                hswpsv(i,j)=fmat(i,j)                                   1128JC97
                fmat(i,j)=0.D0                                          1128JC97
              enddo                                                     1128JC97
           enddo                                                        1128JC97
        endif                                                           1128JC97
       enddo                                                            1128JC97
      endif                                                             1128JC97
C
C     Calculations for starting geometry
C
      iopc=5
      if (irepr(5).eq.1) then                                           0811YC97
         if (ioptgr(iopc).eq.1.and.istatu(5).lt.2) then                 0725YC97
            call geoin(iopc,ircm)
            if (ircm.eq.1) call rchmul(iopc)
         else
            imultipl(5)=nedeg(13)                                       0725YC97
                 do inatnop=1,n3tm
                 x(inatnop)=xr(inatnop,5)
                 enddo
         endif
c   
c   For avoiding problems related to reorientation, we will translate 
c   origin of coordinates to the center of masses of the optimized 
c   saddle point (energy, gradients and hessians will not change, but
c   it will be safer for the calculation of the reaction path).
c
           totm=0.D0
           do i=1,3
              xcm(i)=0.D0
           enddo
           do k=1,natom
             ams=amass(k*3)**2.0D0
             totm=totm+ams
             do l=1,3
               xcm(l)=xcm(l)+x((k-1)*3+l)*ams
             enddo
           enddo
           do k=1,natom
               do l=1,3
                  x((k-1)*3+l)=x((k-1)*3+l)-xcm(l)/totm
               enddo
           enddo
c
c   Now, it calculates the gradients and hessian for the saddle point
c
           if (istatu(5).lt.4) then                                     0725YC97
                 if (isavect.ne.2) then
                 call secin(iopc)
                 endif
           endif                                                        0725YC97
                 estar=v
C
C     saves the result for the following calculations on these points
C
       enstsv=estar
       do i=1,n3tm
       gestsv(i)=x(i)
       xr(i,5)=x(i)
       x(i)=0.D0
       fsstsv(i)=DX(i)
       dx(i)=0.D0
        do j=1,n3tm
        hsstsv(i,j)=fmat(i,j)
        fmat(i,j)=0.D0
        enddo
       enddo
      endif
      endif                                                             0811YC97
c
      return
1001  format (3x,'NWChemrate error: SUPERMOL keyword is only supported',
     *' along with the',/,3x,'following options:',//,7x,'OHOOK option ',
     *'in OPTTS and OPTMIN',/,7x,'HHOOK option in HESSCAL',/,7x,'HAR',
     *'MONIC or HINROT approximation for vibrational modes')
      end                                           
c
c*****
c
c***********************************************************************
      subroutine prepj(jtype)
c***********************************************************************
c
c     This routine is the species dependent
c     driver for all preparation routines needed to set up an electronic 
c     structure calculation.  Any initializations that are dependent
c     on wheter the reactants, products, saddle point, or generalized
c     transition state is computed should be done here.  JTYPE specifies
c     the species:
c
c           JTYPE               Species
c             1                 Reactant #1
c             2                 Reactant #2
c             3                 Product #1
c             4                 Product #2
c             5                 Saddle point
c             6                 Generalized transition states
c             7                 Reactant well
c             8                 Product well
c
c     This routine is called after each respective call to stvarj.
c
c
      implicit double precision (a-h,o-z)
c
      include 'param.inc'
      include 'percon.inc'
      include 'common.inc'
      include 'nwrate.inc'
      common /ncgcom/ iop
      common /eshscm/ fmat(n3tm,n3tm)
      if (jtype.ne.6) then
         call stlgsj(jtype)
         call stvarj(jtype)
      end if
      iop=jtype
      return
      end
c
c*****
c
c***********************************************************************
       SUBROUTINE rchmul(jtype)
c***********************************************************************
c
c     This subroutine reads the charge and multiplicity from a
c     NWChem output file
c     Modified version of Gaussrate version, MAI Nov. 12, 2006
c
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      include 'param.inc'
      include 'percon.inc'
      include 'common.inc'
      include 'nwrate.inc'
      integer i,n
      logical findC,findM
      character*80 line,cmult
      character*8  searchC,searchS
      character*5 searchM
C
C     It starts looking for the charge by means of the title
C     'General Information'
C
      OPEN (UNIT=FU85,STATUS='UNKNOWN',FILE='esp.fu85')
C
      searchC=' charge '

C     call rword(nwmeth,icstrt,ierr)
C     icend=len_trim(nwmeth)
C     searchM=nwmeth(icstart+1:icend)//':mult('
C     lsrchM=len_trim(searchM)

      searchM=':mult'
      searchS=':scftype'

      if(debug)write(666,*)'**',searchC,'**'
      if(debug)write(666,*)'**',searchM(1:lsrchM),'**'

c MAI: assuming charge=0 if not found in esp.fu85
c      findC=.true.
      icharge(jtype)=0
      findC=.false.
      findM=.true.

      do while(.true.)
         read(fu85,'(a80)',end=999)line
         if(index(line,searchC).ne.0)then
C           write(6,*)line
            read(fu85,*)bla
            icharge(jtype)=int(bla)
            findC=.false.
C         elseif(index(line,searchM(1:lsrchM)).ne.0)then
          elseif(index(line,searchM).ne.0) then
            if(debug)write(666,*) line
            read(fu85,*)cmult                                           1002JZ07
            if(index(cmult,'singlet').ne.0) then
               imultipl(jtype)=1
            elseif(index(cmult,'doublet').ne.0) then
               imultipl(jtype)=2
            elseif(index(cmult,'triplet').ne.0) then
               imultipl(jtype)=3
            elseif(index(cmult,'quartet').ne.0) then 
               imultipl(jtype)=4
            elseif(index(cmult,'quintet').ne.0) then 
               imultipl(jtype)=5
            elseif(index(cmult,'sextet').ne.0)  then
               imultipl(jtype)=6
            elseif(index(cmult,'septet').ne.0)  then
               imultipl(jtype)=7
            elseif(index(cmult,'octet').ne.0)   then
                imultipl(jtype)=8
            else
                imultipl(jtype)=icint(cmult)
            endif                                                       1002JZ07
C            read(fu85,*)imultipl(jtype)
            findM=.false.
C search SCFTYPE                                                        1004JZ07
         elseif(index(line,searchS).ne.0.and.index(nwmeth,'dft').eq.0)
     *   then
            read(fu85,'(a4)') scftype(jtype)
         endif
      enddo

 999  continue

      if(debug)write(666,*)findC,findM
      if(findC.or.findM)then
         write(fu6,1000)
         Stop
      endif

      close(fu85)

 1000 format('Error: charge and/or multiplicity not found in NWChem',/,
     &  'output file. Please check NWChem job run. Thanks.')

      end
c
c*****
c
c***********************************************************************
       subroutine rcodnw(icount,slinrd)
c***********************************************************************
c
c     subroutine to read in the input for the routes 
c
c     modified Gaussrate rcodgr by MAI Nov. 10, 2006
c
      implicit double precision (a-h,o-z)
      include 'param.inc'
      include 'percon.inc'
      include 'common.inc'
      include 'nwrate.inc'
      character*80 slinrd(maxlnnw)
      character * 80 string, case, tempchar
      character * 6 qend
      external case
c
      icount=0
c
c   Read the line
c
1     read (fu70,'(a80)') string
      iblnk = 1
c
c   Check if it is the 'end' line
c
      i=1
      do while (iblnk.eq.1.and.i.le.80)
         if (string(i:i).ne.' ') iblnk=0
         i=i+1
      enddo
      tempchar=string
      qend=case(tempchar(i-1:i+4))
c 
c   if it is not the end line, add the readed line to the array containing 
c   the input. Otherwise, finish reading
c
      if (qend.ne.'nwrend') then
         slinrd(icount+1)=string
         icount=icount+1
         if (icount.eq.maxlnnw) then
             write(fu6,*)'Input too long. Maximum:', maxlnnw,' linesi.'
             write(fu6,*)'Increase maxlnnw in nwrate.inc and recompile.'
             stop
         endif
         goto 1
      else
         return
      endif
      return
      end
c
c*****
c
c***********************************************************************
      SUBROUTINE redinf(i,fmat,ierr)
c***********************************************************************
C
C   This subroutine reads the information about reactants, products and
C   saddle point and gives that information to polyrate, avoiding the
C   recalculation of properties that were previously calculated.
C   That information was stored in the matrices that go through the
C   "savcom" common.
C    The information on wells has also been added (JC, 11/28/97)
C
C   The subroutine checks that the geometry is the same that the one it
C   has stored; otherwise, ierr=1 and Polyrate will calculate the
C   information elsewhere.
C
      implicit double precision(a-h,o-z)
      include 'param.inc'
      include 'percon.inc'
      include 'common.inc'
      include 'nwrate.inc'
      dimension fmat(n3tm,n3tm)
      dimension storcm(3),ancm(3)
      ierr=0
      alimg=1.0D-12
      do l=1,3
      storcm(l)=0.D0
      ancm(l)=0.D0
      enddo
      totm=0.D0
C
C     First, check the input geometry against the stored one, measuring
C     the distance between the atoms and the center of mass in both
C     cases (this is only valid for stationary points, since along the
C     reaction coordinate, global translation or rotation will cause
C     problems). If isavect=1, then the check is made for the global
C     positions instead of the positions with respect to the center
C     of masses. If isavect=2, no check is done. This is the safer option,
C     to be used when calculating anharmonicities or numerical second
C     derivatives
C
      if (isavect.eq.2) then
               ierr=1
               return
      endif
      if (isavect.eq.1) then
          if (i.ne.5) then                                              1128JC97
            do j=1,natom
              do k=1,natom
                if (j.eq.iatsv(k,i)) then
                    do l=1,3
                      if (i.lt.3) then
                         dif = x((j-1)*3+l)-geresv((j-1)*3+l)
                      else
                         dif = x((j-1)*3+l)-geprsv((j-1)*3+l)
                      endif
                      if (abs(dif).gt.alimg) then
                         ierr=1
                         return
                      endif
                    enddo
                 endif
              enddo
            enddo
          else
            do j=1,natom
                    do l=1,3
                         dif = x((j-1)*3+l)-gestsv((j-1)*3+l)
                      if (abs(dif).gt.alimg) then
                         ierr=1
                         return
                      endif
                    enddo
            enddo
          endif
       endif
      if (i.ne.5) then                                                  1128JC97
          do 1 j=1,natom
           do 2 k=1,natom
             if (j.eq.iatsv(k,i)) then
                 ams=amass(j*3)**2.0D0
                 totm=totm+ams
                do l=1,3
c
c   Calculate the center of mass of the current position (ancm) and of
c   the stored positions (storcm).
c
                 ancm(l)=ancm(l)+x((j-1)*3+l)*ams
                 if (i.lt.3) then
                  storcm(l)=storcm(l)+geresv((j-1)*3+l)*ams
                 else if (i.le.4) then                                  1128JC97
                  storcm(l)=storcm(l)+geprsv((j-1)*3+l)*ams
                 else if (i.eq.7) then                                  1128JC97
                  storcm(l)=storcm(l)+gewrsv((j-1)*3+l)*ams             1128JC97
                 else if (i.eq.8) then                                  1128JC97
                  storcm(l)=storcm(l)+gewpsv((j-1)*3+l)*ams             1128JC97
                 endif
                enddo
             endif
2          continue
1         continue
          do 3 j=1,natom
           do 4 k=1,natom
             if (j.eq.iatsv(k,i)) then
                dif=0.D0
                disnx=0.D0
                dissx=0.D0
                do l=1,3
c
c   Calculate the distance between the atoms and the center of mass
c   in both cases and compare them
c
                  disnx=disnx+(x((j-1)*3+l)-ancm(l)/totm)**2.0D0
                  if (i.lt.3) then
                   dissx=dissx+(geresv((j-1)*3+l)-storcm(l)/totm)**2.0D0
                  else if (i.le.4) then                                 1128JC97
                   dissx=dissx+(geprsv((j-1)*3+l)-storcm(l)/totm)**2.0D0
                  else if (i.eq.7) then                                 1128JC97
                   dissx=dissx+(gewrsv((j-1)*3+l)-storcm(l)/totm)**2.0D01128JC97
                  else if (i.eq.8) then                                 1128JC97
                   dissx=dissx+(gewpsv((j-1)*3+l)-storcm(l)/totm)**2.0D01128JC97
                  endif
                enddo
                  dif=sqrt(disnx)-sqrt(dissx)
                    if (abs(dif).gt.alimg) then
                        ierr=1
                        return
                    endif
             endif
4          continue
3         continue
      endif
C
      if (i.eq.5) then                                                  1128JC97
C
C   We have to do the same for the saddle point, although a bit simpler, 
C   since all the atoms are involved
C
          do 5 k=1,natom
            ams=amass(k*3)**2.0D0
            totm=totm+ams
            do l=1,3
c
c   Calculate the center of mass of the current position (ancm) and of
c   the stored positions (storcm).
c
               ancm(l)=ancm(l)+x((k-1)*3+l)*ams
               storcm(l)=storcm(l)+gestsv((k-1)*3+l)*ams
            enddo
5        continue
                dif=0.D0
                disnx=0.D0
                dissx=0.D0
          do 6 k=1,natom
            do l=1,3
c
c   Calculate the distance between the atoms and the center of mass
c   in both cases and compare them
c
                 disnx=disnx+(x((k-1)*3+l)-ancm(l)/totm)**2.0D0
                 dissx=dissx+(gestsv((k-1)*3+l)-storcm(l)/totm)**2.0D0
            enddo
                 dif=sqrt(disnx)-sqrt(dissx)
                 if (abs(dif).gt.alimg) then
                        ierr=1
                        return
                 endif
6         continue
      endif
C
C   If the program is still in this subroutine, it is because the 
C   comparison of geometries indicated that we are at the stored
C   point. Therefore, now it has to read the saved information 
C
      if (i.le.2) then
            v=enresv
            if (isup.eq.1) then
                if (i.eq.1) v=ereac1
                if (i.eq.2) v=ereac2
            endif
            do k=1,n3tm
            x(k)=geresv(k)
            dx(k)=fsresv(k)
               do l=1,n3tm
                   fmat(k,l)=hsresv(k,l)
               enddo
            enddo
      else if (i.le.4) then
            v=enprsv
            if (isup.eq.1) then
                if (i.eq.3) v=eprod1
                if (i.eq.4) v=eprod2
            endif
            do k=1,n3tm
            x(k)=geprsv(k)
            dx(k)=fsprsv(k)
               do l=1,n3tm
                 fmat(k,l)=hsprsv(k,l)
               enddo
            enddo
      else if (i.eq.5) then                                             1128JC97
            v=enstsv
            do k=1,n3tm
            x(k)=gestsv(k)
            dx(k)=fsstsv(k)
               do l=1,n3tm
                 fmat(k,l)=hsstsv(k,l)
               enddo
            enddo
      else if (i.eq.7) then                                             1128JC97
            v=enwrsv                                                    1128JC97
            do k=1,n3tm                                                 1128JC97
            x(k)=gewrsv(k)                                              1128JC97
            dx(k)=fswrsv(k)                                             1128JC97
               do l=1,n3tm                                              1128JC97
                 fmat(k,l)=hswrsv(k,l)                                  1128JC97
               enddo                                                    1128JC97
            enddo                                                       1128JC97
      else if (i.eq.8) then                                             1128JC97
            v=enwpsv                                                    1128JC97
            do k=1,n3tm                                                 1128JC97
            x(k)=gewpsv(k)                                              1128JC97
            dx(k)=fswpsv(k)                                             1128JC97
               do l=1,n3tm                                              1128JC97
                 fmat(k,l)=hswpsv(k,l)                                  1128JC97
               enddo                                                    1128JC97
            enddo                                                       1128JC97
      else                                                              1128JC97
            write (fu6,*) 'Something strange happened in REDINF. It',   1128JC97
     *                    ' entered with jtype=', i                     1128JC97
      endif
      return
      end
c
c*****
c
c***********************************************************************
      subroutine rdnwtop(string,istrt,jtype)
c***********************************************************************
c
c     subroutine to read in a the 'top' part of an NWChem file
c     modified Gaussrate subrutine Nov. 10, 2006, MAI
c
      implicit double precision (a-h,o-z)
      include 'param.inc'
      include 'percon.inc'
      include 'common.inc'
      include 'nwrate.inc'
      character*80 slinrd(maxlnnw)
      character * 80 string, case
      external case
c
       call rcodnw(icount,slinrd)
          nlntop=icount
            do i=1,icount
               nwtop(i)=slinrd(i)
            enddo
c
      return
      end
c
c*****
c
c***********************************************************************
      subroutine rdnwmid(string,istrt,jtype)
c***********************************************************************
c
c     subroutine to read in a the 'middle' part of an NWChem file
c     modified Gaussrate subrutine Nov. 10, 2006, MAI
c
      implicit double precision (a-h,o-z)
      include 'param.inc'
      include 'percon.inc'
      include 'common.inc'
      include 'nwrate.inc'
      character*80 slinrd(maxlnnw)
      character * 80 string, case
      external case
c
       call rcodnw(icount,slinrd)
          nlnmid=icount
            do i=1,icount
               nwmid(i)=slinrd(i)
            enddo
c
      return
      end
c
c*****
c
c***********************************************************************
      subroutine rdnwmethop(string,istrt,jtype)
c***********************************************************************
c
c     subroutine to read in a the 'method options' part of an NWChem file
c     modified Gaussrate subrutine Nov. 10, 2006, MAI
c
      implicit double precision (a-h,o-z)
      include 'param.inc'
      include 'percon.inc'
      include 'common.inc'
      include 'nwrate.inc'
      character*80 slinrd(maxlnnw)
      character * 80 string, case
      external case
c
       call rcodnw(icount,slinrd)
          nlnmethop=icount
            do i=1,icount
               nwmethop(i)=slinrd(i)
            enddo
c
      return
      end
c
c*****
c
c***********************************************************************
      subroutine rdnwscfop(string,istrt,jtype)
c***********************************************************************
c
c     subroutine to read in a the 'SCF block options' part of an NWChem file
c     these options are needed for post-SCF methods (MP2, CCSD, etc)
c     modified Gaussrate subrutine Nov. 10, 2006, MAI
c
      implicit double precision (a-h,o-z)
      include 'param.inc'
      include 'percon.inc'
      include 'common.inc'
      include 'nwrate.inc'
      character*80 slinrd(maxlnnw)
      character * 80 string, case
      external case
c
       call rcodnw(icount,slinrd)
          nlnscf=icount
            do i=1,icount
               nwscfop(i)=slinrd(i)
            enddo
c
      return
      end
c
c*****
c
c***********************************************************************
      subroutine rdnwoptop(string,istrt,jtype)
c***********************************************************************
c
c     subroutine to read in a the 'optimisation options' part of an NWChem file
c     modified Gaussrate subrutine Nov. 10, 2006, MAI
c
      implicit double precision (a-h,o-z)
      include 'param.inc'
      include 'percon.inc'
      include 'common.inc'
      include 'nwrate.inc'
      character*80 slinrd(maxlnnw)
      character * 80 string, case
      external case
c
       call rcodnw(icount,slinrd)
          nlnoptop=icount
            do i=1,icount
               nwoptop(i)=slinrd(i)
            enddo
c
      return
      end
c
c*****
c
c***********************************************************************
       subroutine rgennw(string,iend,istrt)
c***********************************************************************
c
c     subroutine to read in the input in the general section. By
c     now, it only includes the restart option
c
c      This subroutine was used as is from Gaussrate
c
      implicit double precision (a-h,o-z)
      include 'param.inc'
      include 'percon.inc'
      include 'common.inc'
c small change: MAI Nov. 10, 2006
      include 'nwrate.inc'
      character * 80 string
c
c     set the defaults:
c
c     GRRESTART: ON
      irest=1
c
c     RSTTOL: 1.0D-8                                                     0825JC00
      fallout=1.0D-8                                                     0825JC00
      nwproc=1
c
c     read in first keyword in this section
c
      call rlingr(string,istrt,isect,iend)
c
      do while (isect.eq.0.and.iend.eq.0)
c NWRRESTART
c small change: MAI Nov. 10, 2006
         if (string(istrt:istrt+5).eq.'nwrres') then
            irest = 1
c NOGRRESTART
c small change: MAI Nov. 10, 2006
         else if (string(istrt:istrt+7).eq.'nonwrres') then
            irest = 0
c NWPROC
         else if (string(istrt:istrt+5).eq.'nwproc') then
            call rword(string,istrt,ierr)
            if(ierr.ne.1) then
               nwproc = cfloat(string(istrt:80))
            else
               write(fu6,*)'ERROR reading keyword NWPROC in esp.fu70'
               stop
            end if
	    write(procnw,'(I4)')nwproc
c RSTTOL                                                                 0825JC00
         else if (string(istrt:istrt+5).eq.'rsttol') then                0825JC00
            call rword(string,istrt,ierr)                                0825JC00
            if(ierr.ne.1) then                                           0825JC00
               fallout = cfloat(string(istrt:80))                        0825JC00
               write(fu6,*)'WARNING: RSTTOL changed to', fallout         0825JC00
            else                                                         0825JC00
               write(fu6,*)'ERROR reading keyword RSTTOL in esp.fu70'    0825JC00
            stop                                                         0825JC00
            end if                                                       0825JC00
         else
            write(fu6,1200) string(istrt:80)
            stop
         end if
         call rlingr(string,istrt,isect,iend)
      end do
c
1200  format(3x,'Error:  the following is not a valid keyword in the',
     *       ' grgeneral section',/A80)
c
      return
      end
c
c*****
c
c***********************************************************************
      SUBROUTINE rinpnw
c***********************************************************************
C
c
C     This subroutine reads the input of the NWrate options. It
C     starts reading if it is a NWrate restart run. If it is, open
c     the reading file and the writing file.
c
c     Modified version of Gaussrate rinpnw, Nov. 10, 2006, by MAI
c
c
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      include 'param.inc'
      include 'percon.inc'
      include 'common.inc'
      include 'nwrate.inc'
c     character * 80 string,case                                        0701TA03
c     Variable case was not used so it was taken out in Gaussrate 9.1.  0701TA03
      character * 80 string                                             0701TA03
CCMAI - NOT FOR NWRATE -      inlcer=0
CCMAI - NOT FOR NWRATE -      inlbas=0
CCMAI - NOT FOR NWRATE -      do i=1,8                                                          1128JC97
CCMAI - NOT FOR NWRATE -       inlscr(i)=1
CCMAI - NOT FOR NWRATE -       inlfrs(i)=1
CCMAI - NOT FOR NWRATE -       inlene(i)=1
CCMAI - NOT FOR NWRATE -       secrot(i,1) = '#hf/sto-3g units=au fchk nosymm freq=noraman'
CCMAI - NOT FOR NWRATE -       firrot(i,1) = '#hf/sto-3g units=au fchk nosymm force'
CCMAI - NOT FOR NWRATE -       enerot(i,1) = '#hf/sto-3g units=au fchk nosymm'
CCMAI - NOT FOR NWRATE -      enddo

      nlntop=0
      nlnmid=0
      nlnmethop=0
      nlnoptop=0
      nlnscf=0
c
      call rlingr(string,istrt,isect,iend)
c
C     Those error check lines are equivalents to those when reading fu5
c
      if (iend.eq.1) then
         write(fu6,1000)
1000     format(1x,'error:  input file was empty')
         stop
      end if
c
      if (isect.ne.1) then
         write(fu6,1100)
1100     format(1x,'error:  first non-comment, non-blank line must be'
     *             ,' a section header that begins with a *')
         stop
      end if
c
c     Now, it reads the sections in grate input
c
      do while (iend.eq.0)
         j = istrt + 1
c         j = istrt
         if (string(j:j+6).eq.'nwrgene') then
               call rgennw(string,iend,istrt)
         else if (string(j:j+8).eq.'nwrreact1') then
               call rstatnw(string,iend,istrt,1)
         else if (string(j:j+8).eq.'nwrreact2') then
               call rstatnw(string,iend,istrt,2)
         else if (string(j:j+7).eq.'nwrprod1') then
               call rstatnw(string,iend,istrt,3)
         else if (string(j:j+7).eq.'nwrprod2') then
               call rstatnw(string,iend,istrt,4)
         else if (string(j:j+7).eq.'nwrstart') then
               call rstatnw(string,iend,istrt,5)
         else if (string(j:j+6).eq.'nwrpath') then
               call rstatnw(string,iend,istrt,6)
         else if (string(j:j+7).eq.'nwrwellr') then                      1128JC97
               call rstatnw(string,iend,istrt,7)                        1128JC97
         else if (string(j:j+7).eq.'nwrwellp') then                      1128JC97
               call rstatnw(string,iend,istrt,8)                        1128JC97
         else if (string(j:j+8).eq.'nwrcommon') then
               call rstatnw(string,iend,istrt,9)                        1128JC97
         else
               write(fu6,1200) string(j-1:80)
               stop
         end if
      end do
c
1200  format(3x,'Error:The following is not a valid NWR section name',
     *         /A80)
c
      if (irest.eq.1) then
        if(ivrc.eq.1) then                                              0921JZ09
         OPEN (UNIT=FU83,FILE='../esp.fu83')
        else
         OPEN (UNIT=FU83,FILE='./esp.fu83')
        endif
         OPEN (UNIT=FU84,STATUS='NEW',FILE='esp.fu84')
         write(fu6,2001)
      endif
c
2001     format(/,5x,'NWChemrate restart calculation',/)
C
      RETURN
      END
c
c*****
c
c***********************************************************************
      SUBROUTINE rlingr(string,istrt,isect,iend)
c***********************************************************************
C
c     This subroutine is equivalent to rline in interface.f, but a bit
c     modified for reading from unit fu70
c
c      istrt:   location in 80 character string that is the first non-blank
c      isect:   a 1 is returned if the first non-blank character is a *
c               signalling the start of a section
c      iend:    a 1 is returned if the end of file 5 is encountered
c               or NWRNWREND is reached
c      string:  80 character string that is the contents of the current line
c
c      This subroutine was used as is from Gaussrate, except for the last include
c      file. For ease, name left unchanged. By: MAI Nov. 10, 2006.
c
      implicit double precision (a-h,o-z)
c
      include 'param.inc'
      include 'percon.inc'
      include 'common.inc'
      include 'nwrate.inc'
c
      character * 80 string,case
      external case
c
1000  format(A80)
c
c       initialize variables
c
      isect = 0
      iend = 0
      iblnk = 1
c
c     read in next line in file 5 - find first non-blank character and
c     determine if it is a comment (# sign signals a comment).  Once a
c     comment is found the rest of the line is skipped (by setting the
c     counter to the last character [i=80])
c
      do while (iblnk.eq.1)
 2       read(fu70,1000,end=9999) string
         if((index(string,'NWRNWREND').ne.0).or.
     *     (index(string,'nwrnwrend').ne.0))goto 9999
         i = 1
         do while (iblnk.eq.1.and.i.le.80)
            if (string(i:i).ne.' ') then
               if (string(i:i).ne.'#') then
                   iblnk = 0
                else
                   i = 80
                end if
            end if
            i = i+1
            if((i.eq.80).and.(iblnk.eq.1))goto 2
         end do
       end do
c
c      set location of first non-blank character
       istrt = i-1
c
c      check if it is a section header (first character a *)
c
       if (string(istrt:istrt).eq.'*') isect=1
c
c      strip of any trailing comments on the line
c
       do 1 j=istrt,80
1        if (string(j:j).eq.'#') string(j:80) = ' '
c
c      change to lower case
c
       string = case(string)
       return
9999   iend=1
       return
       end
c
c*****
c
c***********************************************************************
       subroutine rstatnw(string,iend,istrt,jtype)
c***********************************************************************
c
c     subroutine to read in the input for all the stationary points
c
c     modified version of Gaussrate equivalent, Nov. 10, 2006 by MAI
c
      implicit double precision (a-h,o-z)
      include 'param.inc'
      include 'percon.inc'
      include 'common.inc'
      include 'nwrate.inc'
      character * 80 string
      integer tmp,tmp2
c
c     Jtype = 1  first reactant
c     Jtype = 2  second reactant
c     Jtype = 3  first product
c     Jtype = 4  second product
c     Jtype = 5  saddle point or starting stationary point
c     Jtype = 6  all the points along the path
c     Jtype = 7  reactant well
c     Jtype = 8  product well
c     Jtype = 9  all the stationary points and points along the path
c
c     read in first keyword in this section
c
      call rlingr(string,istrt,isect,iend)
c
      do while (isect.eq.0.and.iend.eq.0)
c
C GROPT
         if (string(istrt:istrt+4).eq.'nwropt') then
            write (fu6,1150)                                            1128JC97
            stop                                                        1128JC97
C NOGROPT
         else if (string(istrt:istrt+6).eq.'nonwropt') then
            write (fu6,1150)                                            1128JC97
            stop                                                        1128JC97
C CHARGE
         else if (string(istrt:istrt+5).eq.'charge') then
            call rword(string,istrt,ierr)
            if(ierr.ne.1) then
              if (jtype.ne.9.and.jtype.ne.6) then                       1128JC97
               icharge(jtype) = icint(string(istrt:80))
              else 
               write(fu6,1050)
               stop
              endif
            else
               write(fu6,1100)
               stop
            end if
C MULTIPLICITY
         else if (string(istrt:istrt+5).eq.'multip') then
            call rword(string,istrt,ierr)
            if(ierr.ne.1) then
              if (jtype.ne.9.and.jtype.ne.6) then                       1128JC97
                 imultipl(jtype) = icint(string(istrt:80))
              else 
                 write(fu6,1050)
                 stop
              endif
            else
               write(fu6,1100)
               stop
            end if
C SCTTYPE
         elseif (string(istrt:istrt+6).eq.'scftype') then               1004JZ07
            call rword(string,istrt,ierr)
            if(ierr.ne.1) then
              if (jtype.ne.9.and.jtype.ne.6) then                       1128JC97
                 scftype(jtype) = string(istrt:istrt+4)
              else
                 write(fu6,1050)
                 stop
              endif
            else
               write(fu6,1100)
               stop
            end if
C NWRMETHOD (only allowed in NWRCOMMON)
         elseif (index(string,'nwrmethod').ne.0) then
            if (jtype.eq.9) then
               tmp=index(string,'nwrmethod')+9
c	       tmp2=len_trim(string(tmp:80)
c	       if(tmp.gt.9)then
c	          write(fu6,1060)
c		  stop
c	       endif
	       nwmeth=string(tmp+1:tmp+10)
	       if((index(nwmeth,'mcscf').ne.0).or.
     *             (index(nwmeth,'tce').ne.0).or.
     *             (index(nwmeth,'pspw').ne.0).or.
     *             (index(nwmeth,'td').ne.0))then
	          write(fu6,1070)nwmeth
		  stop
	       endif
            else
              write(fu6,1250)
              stop
            endif
C NWRTOP (only allowed in NWRCOMMON)
         elseif (index(string,'nwrtop').ne.0)then
            if (jtype.eq.9) then                                        1128JC97
              call rdnwtop(string,istrt,jtype)
            else
              write(fu6,1250)
              stop
            endif
C NWRMID (only allowed in NWRCOMMON)
         elseif (index(string,'nwrmid').ne.0)then
            if (jtype.eq.9) then                                        1128JC97
              call rdnwmid(string,istrt,jtype)
            else
              write(fu6,1250)
              stop
            endif
C NWRMETHOP (only allowed in NWRCOMMON)
         elseif (index(string,'nwrmethop').ne.0)then
            if (jtype.eq.9) then                                        1128JC97
              call rdnwmethop(string,istrt,jtype)
            else
              write(fu6,1250)
              stop
            endif
C NWRSCFOP (only allowed in NWRCOMMON)
         elseif (index(string,'nwrscfop').ne.0)then
            if (jtype.eq.9) then                                        1128JC97
              call rdnwscfop(string,istrt,jtype)
            else
              write(fu6,1250)
              stop
            endif
C NWROPTOP (only allowed in NWRCOMMON)
         elseif (index(string,'nwroptop').ne.0)then
            if (jtype.eq.9) then                                        1128JC97
              call rdnwoptop(string,istrt,jtype)
            else
              write(fu6,1250)
              stop
            endif
         else
            write(fu6,1200) string(istrt:80)
            stop
         end if
         call rlingr(string,istrt,isect,iend)
      end do
c
 1050  format(3x,'Error:  MULTIPLICITY, CHARGE, and SCFTYPE can not be',
     *       ' in NWRCOMMON or NWRPATH sections',/)
 1100  format(3x,'Error reading CHARGE or MULTIPLICITY or SCFTYPE',/)
 1150  format(3x,'Error:  NWROPT and NONWROPT are not longer valid',
     *       ' keywords',/)
 1200  format(3x,'Error:  the following is not a valid keyword in',
     *       ' this section',/A80)
 1250  format(3x,'Error:  NWRMETHOD, NWRTOP, NWRMID, NWRMETHOP,',/,
     *  'NWRSCFOP and NWROPTOP are only allowed in the NWRCOMMON',
     *  ' section',/)
 1060 format(3x,'Error: length of method too long, max 10 characters',/)
 1070 format(3x,'Error: method ',A10,'not yet supported by NWCHEMRATE.',
     *   /,'The required options are too complicated to be cleanly',/,
     *   'passed between NWRate and NWChem at this point.')
c
      return
      end
c
c*****
c
c***********************************************************************
      subroutine hhook(xii,fmat,sstp,nend,mw,icent,iproc)
c***********************************************************************
c
c     This routine optionally accepts the mass-weighted cartesian 
c     coordinates and transforms them to cartesian coordinates (mw=1).
c     The cartesian coordinates are used when computing the energy and
c     second derivatives of the energy with respect to each cartesian 
c     coordinate.  If the mass-weighted option was chosen (mw=1), the 
c     coordinates and first derivatives are transformed back to mass-
c     weighted at the end.  
c
      implicit double precision (a-h,o-z)
c
      include 'param.inc'
      include 'percon.inc'
      include 'common.inc'
      include 'nwrate.inc'
      common /ncgcom/ iop
      dimension xii(n3tm), fmat(n3tm,n3tm), tmpfmat(n3tm,n3tm)
      dimension bup(n3tm)
      character*4 scftypej
c     
c
c     Optionally convert the mass-weighted coordinates to cartesian
c
      if (mw.eq.1) call trans(2,n3,amass,x,dx)
c
c     If the hessian is calculated using polyrate:
c
      if (ispot.eq.0) then
          if (lgs(25).eq.0) then
             if (icent.eq.0) then
                 do 75 i=1,n3
 75                 bup(i) = x(i)
                 call seccen (bup,fmat,mw,sstp)
                 goto 1
             endif
             if (icent.eq.1) then
                 call seccep (mw,sstp,nend)
                 goto 1
             endif
          else if (lgs(25).eq.1) then
                 call deriv2(nend)
                 goto 1
          else if (lgs(25).eq.2) then
                 call derv24(nend)
                 goto 1
          endif
       endif
c
c     If it is calculated using Gaussian:
c     If it is a Grate restart, read the energy, 1st and 2nd der.
c     
      irout=3
      if (irest.eq.1) call grest(irout,fmat)
      if (irout.eq.0) return
c
c     If the calculation corresponds to a stationary point, check if it has
c     already been calculated and stored in redinf.
c
        if (iop.eq.6) then                                              1128JC97
            i=5
            nacat=natom
        else 
            if (iop.eq.5) then
                nacat=natom
            else
                nacat=nratom(iop)
            endif
            i=iop
            call redinf(iop,fmat,ierr)
            if (ierr.eq.0) goto 149 
        endif
               ich=icharge(i)
               imul=imultipl(i)
               scftypej=scftype(i)
               do 119 indfmex=1,n3tm
                do 129 indfmey=1,n3tm
                 fmat(indfmex,indfmey)=0.D0
129             continue
119            continue
C              call escsec(natoms,ich,imul,natom,label,
C    *iatsv,v,x,dx,fmat,iop,nacat,nratom)
               call escsec(natoms,ich,imul,scftypej,natom,label,        1004JZ07
     *iatsv,v,x,dx,fmat,iop,nacat,nratom)
c
c     Reorder the stored hessian (if we are dealing with reactants or
c     products) and convert into mass-weighted
c
149       if (iop.ne.5.and.iop.ne.6) then                               1128JC97
            do l=1,n3tm
            do m=1,n3tm
                tmpfmat(l,m)=fmat(l,m)
                fmat(l,m)=0.0D0
            enddo
            enddo
            do l=1,nratom(iop)
            do m=1,nratom(iop)
                  do k=1,3
                  do n=1,3
                     innx=3*(iatsv(l,iop)-1)+k
                     inny=3*(iatsv(m,iop)-1)+n
                     inox=3*(l-1)+k
                     inoy=3*(m-1)+n
                          fmat(inox,inoy)=tmpfmat(innx,inny)/
     *                     amass(ind(inox))/amass(ind(inoy))
                  enddo
                  enddo
              enddo
              enddo
           else
c
c   If it is the saddle point, we only have to convert the Hessian
c   into mass-weighted
c
c     for non-equilibrium solvation, need to add an extra harmonic term
c
            nend = natom*3
c
            if (LBATH.AND.LFOPT) THEN
              nend = nend + 1
              F1 = PI*PI*REDM/(16.0d0*FRICT)
              F2 = F1/FRICT
              DO I = 1,NEND-1
                 FMAT(NEND,I) = -F2*DIFFU(I)
                 FMAT(I,NEND) = FMAT(NEND,I)
              ENDDO
              FMAT(NEND,NEND)=F2
              DO I = 1,NEND-1
               DO J = 1,I
                 FMAT(I,J) = FMAT(I,J)+F2*DIFFU(I)*DIFFU(J)
                 FMAT(J,I) = FMAT(I,J)
               ENDDO
              ENDDO
            endif
c              do l=1,natom*3
c              do m=1,natom*3
              do l=1,nend
              do m=1,nend
                fmat(l,m)=fmat(l,m)/amass(ind(l))/amass(ind(m))
              enddo
              enddo
           endif
c
c     Put the appropriate zero of energy
c
      if (lezer0) v = v-ezer0   
c
c   calculate the derivative and energy, should be in Cartesian coord
c
      IF (LBATH) THEN
        CALL EFFBATH(2)
      ENDIF
c
 
c
c     Transfer back to mass-weighted coordinates
1     if (mw.eq.1) call trans(1,n3,amass,x,dx)
c
c     If it is a Grate restart, write the energy, 1st and 2nd der.
c     If ispot=0, the calculations of first derivatives or/and energies were 
c     already stored and we don't need to save the information about fmat.
c
      if (ispot.ne.0) then
        if (irest.eq.1) then
          irout=7
          call grest(irout,fmat)
        endif
      endif
c
      return
      end                                           
c
c*****
c
c***********************************************************************
      SUBROUTINE setup
c***********************************************************************
C
C   This subroutine writes a title for the polyrate long output and
C   call the subroutine for reading NWrate input
C
C   Modified Gaussrate version, Nov. 10, 2006, by MAI
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      include 'param.inc'
      include 'percon.inc'
      include 'common.inc'
      include 'nwrate.inc'
C
      if(ivrc.eq.1) then                                                0921JZ09
        open (unit=fu70,status='old',file='../esp.fu70')
      else
        open (unit=fu70,status='old',file='./esp.fu70')
      endif
      call rinpnw
      close (fu70)
C
         WRITE (FU6,100) 
C
100   FORMAT (/,2X,T5,'NWChem version 5.0 will carry out the electronic'
     *        /,2X,T5,'structure calculations needed for obtaining the'
     *        /,2X,T5,'required data for the POLYRATE calculation',/)
C
C     Check compatibility of options and set gropt
C
      if (isup.eq.0) then
         write (fu6,200)
         stop 'setup 1'
      endif
c
c     OHOOK optimization:
c
      if (igpot.eq.1) then                                              1203JC97
         do 10 i=1,8                                                    1203JC97
           if (i.eq.6) goto 10                                          1203JC97
           if (irepr(i).eq.0) goto 10                                   1203JC97
c
c     check all the systems to be optimized have initgeo=hooks
c     and set ioptgr=1
c
           if (istatu(i).eq.0) then                                     1203JC97
              if (initg(i).eq.1) then                                   1203JC97
                  ioptgr(i) = 1                                         1203JC97
              else                                                      1203JC97
                  write (fu6,300) i                                     1203JC97
                  stop 'setup 2'                                        1203JC97
              endif                                                     1203JC97
            else                                                        1203JC97
c
c     check all the systems not to be optimized have initgeo=geom 
c     and set ioptgr=0
c
              if (initg(i).eq.0) then                                   1203JC97
                  ioptgr(i) = 0                                         1203JC97
              else                                                      1203JC97
                  write (fu6,400) i                                     1203JC97
                  stop 'setup 3'                                        1203JC97
              endif                                                     1203JC97
            endif                                                       1203JC97
10       continue                                                       1203JC97
c
c     BFGS or NH optimizations
c
      else                                                              1203JC97
         do 20 i=1,8                                                    1203JC97
           if (i.eq.6) goto 20                                          1203JC97
           if (irepr(i).eq.0) goto 20                                   1203JC97
c
c     check all the systems have initgeo=geom 
c
           if (initg(i).eq.0) then                                      1203JC97
                  ioptgr(i) = 0                                         1203JC97
           else                                                         1203JC97
                  write (fu6,500) i                                     1203JC97
                  stop 'setup 3'                                        1203JC97
           endif                                                        1203JC97
20       continue                                                       1203JC97
      endif                                                             1203JC97
C
c
200   FORMAT (/,2X,T5,'Error: NWChemrate requires the no-supermolecule'
     *        /,2X,T5,'mode. The program will stop',/)
300   FORMAT (/,2x,T5,'Error: OHOOK has been selected for the '         1203JC97
     *        /,2X,T5,'optimization, and the species ', I3,' is going'  1203JC97
     *        /,2X,T5,'to be optimized, so the geometry should be'      1203JC97
     *        /,2X,T5,'given in HOOKS (INITGEO=HOOKS).'/)               1203JC97   
400   FORMAT (/,2x,T5,'Error: OHOOK has been selected for the '         1203JC97   
     *        /,2X,T5,'optimization, and the species ', I3,' is not '   1203JC97   
     *        /,2X,T5,'going to be optimized, so the geometry should '  1203JC97     
     *        /,2X,T5,'be given in FU5 (INITGEO=GEOM).'/)               1203JC97 
500   FORMAT (/,2x,T5,'Error: BFGS or NR has been selected for the '    1203JC97   
     *        /,2X,T5,'optimization, so for the species ', I3,' the'    1203JC97   
     *       /,2X,T5,'geometry should be given in FU5 (INITGEO=GEOM).'/)1203JC97 
C
      RETURN
      END
c
c*****
c
c***********************************************************************
      SUBROUTINE store(V,COORD,DX,FMAT,INDOP)
c***********************************************************************
C
C    This subroutine checks if the first derivative calculation is going 
C    to be carried out on a point where the second derivative has been
C    calculated right before this calcularion. If it is, it reads the
C    first derivative instead of calculating it, so that computing time
C    is saved.
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      include 'param.inc'
      include 'percon.inc'
      include 'nwrate.inc'
      COMMON /STRCOM/ STOREV,STORECOORD(N3TM),STOREDX(N3TM),
     *STOREFMAT(N3TM,N3TM)
      DIMENSION COORD(N3TM),DX(N3TM),FMAT(N3TM,N3TM)
      difall=1.0D-10
      if (indop.eq.1) then
         storev=v
         do instr=1,n3tm
            storecoord(instr)=coord(instr)
            storedx(instr)=dx(instr)
              do jnstr=1,n3tm
               storefmat(instr,jnstr)=fmat(instr,jnstr)
              enddo
         enddo
      else
        do incont=1,n3tm
             dif=COORD(incont)-STORECOORD(incont)
             if (abs(dif).gt.difall) goto 101
        enddo
        v=storev
        do incont=1,n3tm
           dx(incont)=storedx(incont)
        enddo
         indop=1
      endif
101   return
      end
c
c ***************************************************************************
        function Ucase (string,length)
c ***************************************************************************
c
c   Function which takes a string of 80 characters and converts the 
c   lower case letters in the string to upper case letters
c
c   Modification of Polyrate 9.4.2 function case, MAI Nov. 15, 2006
c
      implicit double precision (a-h,o-z)
         character*80 string, Ucase
         character * 1 xlett
c
         do 10 i = 1, length
               xlett = string(i:i)
               itry = ichar (xlett)
                if (xlett .ge. 'a' .and. xlett .le. 'z') then 
                    itry = itry - 32
                    string (i:i) = char (itry)
                endif
   10     continue
c
                Ucase  = string 
c
          return
          end
C
